package x64_test

import (
	"testing"

	"git.akyoto.dev/cli/q/src/build/arch/x64"
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/go/assert"
)

func TestMoveRegisterNumber(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Number   int
		Code     []byte
	}{
		{x64.RAX, 0x7FFFFFFF, []byte{0xB8, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.RCX, 0x7FFFFFFF, []byte{0xB9, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.RDX, 0x7FFFFFFF, []byte{0xBA, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.RBX, 0x7FFFFFFF, []byte{0xBB, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.RSP, 0x7FFFFFFF, []byte{0xBC, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.RBP, 0x7FFFFFFF, []byte{0xBD, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.RSI, 0x7FFFFFFF, []byte{0xBE, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.RDI, 0x7FFFFFFF, []byte{0xBF, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.R8, 0x7FFFFFFF, []byte{0x41, 0xB8, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.R9, 0x7FFFFFFF, []byte{0x41, 0xB9, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.R10, 0x7FFFFFFF, []byte{0x41, 0xBA, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.R11, 0x7FFFFFFF, []byte{0x41, 0xBB, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.R12, 0x7FFFFFFF, []byte{0x41, 0xBC, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.R13, 0x7FFFFFFF, []byte{0x41, 0xBD, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.R14, 0x7FFFFFFF, []byte{0x41, 0xBE, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x64.R15, 0x7FFFFFFF, []byte{0x41, 0xBF, 0xFF, 0xFF, 0xFF, 0x7F}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("mov %s, %x", pattern.Register, pattern.Number)
		code := x64.MoveRegisterNumber32(nil, pattern.Register, uint32(pattern.Number))
		assert.DeepEqual(t, code, pattern.Code)
	}
}

func TestMoveRegisterRegister(t *testing.T) {
	usagePatterns := []struct {
		Left  cpu.Register
		Right cpu.Register
		Code  []byte
	}{
		{x64.RAX, x64.R15, []byte{0x4C, 0x89, 0xF8}},
		{x64.RCX, x64.R14, []byte{0x4C, 0x89, 0xF1}},
		{x64.RDX, x64.R13, []byte{0x4C, 0x89, 0xEA}},
		{x64.RBX, x64.R12, []byte{0x4C, 0x89, 0xE3}},
		{x64.RSP, x64.R11, []byte{0x4C, 0x89, 0xDC}},
		{x64.RBP, x64.R10, []byte{0x4C, 0x89, 0xD5}},
		{x64.RSI, x64.R9, []byte{0x4C, 0x89, 0xCE}},
		{x64.RDI, x64.R8, []byte{0x4C, 0x89, 0xC7}},
		{x64.R8, x64.RDI, []byte{0x49, 0x89, 0xF8}},
		{x64.R9, x64.RSI, []byte{0x49, 0x89, 0xF1}},
		{x64.R10, x64.RBP, []byte{0x49, 0x89, 0xEA}},
		{x64.R11, x64.RSP, []byte{0x49, 0x89, 0xE3}},
		{x64.R12, x64.RBX, []byte{0x49, 0x89, 0xDC}},
		{x64.R13, x64.RDX, []byte{0x49, 0x89, 0xD5}},
		{x64.R14, x64.RCX, []byte{0x49, 0x89, 0xCE}},
		{x64.R15, x64.RAX, []byte{0x49, 0x89, 0xC7}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("mov %s, %s", pattern.Left, pattern.Right)
		code := x64.MoveRegisterRegister64(nil, pattern.Left, pattern.Right)
		assert.DeepEqual(t, code, pattern.Code)
	}
}
