package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/cli/q/src/build/errors"
	"git.akyoto.dev/cli/q/src/build/token"
)

// ExecuteRegisterRegister performs an operation on two registers.
func (f *Function) ExecuteRegisterRegister(operation token.Token, destination cpu.Register, source cpu.Register) error {
	switch operation.Kind {
	case token.Add, token.AddAssign:
		f.RegisterRegister(asm.ADD, destination, source)

	case token.Sub, token.SubAssign:
		f.RegisterRegister(asm.SUB, destination, source)

	case token.Mul, token.MulAssign:
		f.RegisterRegister(asm.MUL, destination, source)

	case token.Div, token.DivAssign:
		f.RegisterRegister(asm.DIV, destination, source)

	case token.Mod, token.ModAssign:
		f.RegisterRegister(asm.MODULO, destination, source)

	case token.And, token.AndAssign:
		f.RegisterRegister(asm.AND, destination, source)

	case token.Or, token.OrAssign:
		f.RegisterRegister(asm.OR, destination, source)

	case token.Xor, token.XorAssign:
		f.RegisterRegister(asm.XOR, destination, source)

	case token.Equal, token.NotEqual, token.Less, token.LessEqual, token.Greater, token.GreaterEqual:
		f.RegisterRegister(asm.COMPARE, destination, source)

	case token.Assign:
		f.RegisterRegister(asm.MOVE, destination, source)

	default:
		return errors.New(&errors.InvalidOperator{Operator: operation.Text(f.File.Bytes)}, f.File, operation.Position)
	}

	return nil
}
