package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/errors"
	"git.akyoto.dev/cli/q/src/build/expression"
	"git.akyoto.dev/cli/q/src/build/token"
)

// ExpressionToMemory puts the result of an expression into the specified memory address.
func (f *Function) ExpressionToMemory(node *expression.Expression, memory asm.Memory) error {
	if node.IsLeaf() && (node.Token.Kind == token.Number || node.Token.Kind == token.Rune) {
		number, size, err := f.Number(node.Token)

		if err != nil {
			return err
		}

		if size != memory.Length {
			return errors.New(&errors.NumberExceedsBounds{Number: number, ExpectedSize: memory.Length, Size: size}, f.File, node.Token.Position)
		}

		f.MemoryNumber(asm.STORE, memory, number)
		return nil
	}

	tmp := f.NewRegister()
	defer f.FreeRegister(tmp)
	err := f.ExpressionToRegister(node, tmp)
	f.MemoryRegister(asm.STORE, memory, tmp)
	return err
}
